#!perl
#
#    A little Perl program to fix broken TLIB Journal files created
# by TLIB 5.53 prior to 5.53L.
#
# -Dave Burton   6/26/2002


# immediate output of debug prints
$| = 1;

@INC = (".","..");
# print "before 'require splitcsv.pl'\n";
require ".\\splitcsv.pl";
# print "after 'require splitcsv.pl'\n";

$verbose = 0;
$changed_only = 0;
$v4_convert = 0;
while (($#ARGV >= 0) && ($ARGV[0] =~ /^\-[wf4]$/)) {
   if ('-w' eq $ARGV[0]) {
      $verbose = 1;
   } elsif ('-f' eq $ARGV[0]) {
      $changed_only = 1;
   } elsif ('-4' eq $ARGV[0]) {
      $v4_convert = 1;
   }
   shift @ARGV;
}

$num_args = $#ARGV+1;

if (($num_args > 3) || ($num_args < 2)) {
 print "JourFix -- Fix TLIB Journal file created by buggy versions of TLIB.\n" .
       "Copyright 2002, Burton Systems Software.  www.burtonsys.com, 1-919-481-0149.\n" .
       "\n" .
       "TLIB 5.53 prior to 5.53L (and TLIB 5.52h/beta) had a bug which caused the 7th\n" .
       "field of the journal file to usually be omitted.  Fortunately, that field is\n" .
       "almost interchangable with the 10th field, since they are both just variations\n" .
       "of the source file name.  This program fixes such journal files by using the\n" .
       "10th field to replace the missing 7th field.\n" .
       "\n" .
       "Also, TLIB 5.00 prior to 5.00i had a bug which caused comment-only records\n" .
       "to have one null field too few (i.e., the comment was field 12 instead of\n" .
       "field 13).  This program fixes those records, too.\n" .
       "\n" .
       "Usage (under Windows 9x/Me/NT/2K/XP, or under MS-DOS, respectively):\n" .
       "      perl4 jourfix.pl {options} in.dat out.dat\n" .
       "      perl4s jourfix.pl {options} in.dat out.dat\n" .
       "      (or, if you have Perl 5 installed, you can it)\n" .
       "\n" .
       "Where:\n" .
       "      {options} is optional.  It can be:\n" .
       "         -w to view warnings about unrecognized input lines, and/or\n" .
       "         -f to delete the output file unless at least 1 line was fixed, and/or\n" .
       "         -4 to convert TLIB 4.12 journal files to v5 (13 field) format.\n" .
       "      in.dat is the journal.dat file that needs to be fixed, and\n" .
       "      out.dat is the fixed journal.dat file, created by this program\n" .
       "\n" .
       "Note: Perl4s for MS-DOS does not support long file names.\n";
       exit 1;
}

$infile = shift @ARGV;
$infile =~ tr/a-z/A-Z/;  # upper-case

$outfile = shift @ARGV;
# $outfile =~ tr/a-z/A-Z/;

if (!open(INP,$infile)) {
   printf "ERROR: could not find '%s', $!\n", $infile;
   exit 1;
}
unlink $outfile;  # unlink is just in case of buggy MARS_NWE file system that won't truncate existing files
if (!open(OUTP,">$outfile")) {
   printf "ERROR: could not create '%s', $!\n", $outfile;
   exit 1;
}

print "infile='$infile' outfile='$outfile'\n";

$numrecs = 0;
$numfixedrecs = 0;
$numfixedcmtrecs = 0;
$numV4converted = 0;
while (<INP>) {
   $numrecs++;
   chop;
   $_ =~ s/[\r\n]*$//;  # because 16-bit Perl lack chomp (to strip both cr and lf), we make sure here
   $fixed = 0;
   $v4 = 0;

   if ('' ne $_) {

      if ($_ =~ /^\,\,\,\,\,\,\,\,\,/) {
         # additional comment record (starts with a bunch of commas)
         if ($_ =~ /^\,\,\,\,\,\,\,\,\,\,\,"/) {
            # TLIB 5.0 prior to 5.00i erroneously generated only 11 leading commas
            # for additional comment records.  Oops.  This repairs such records.
            $_ = ',' . $_;
            $numfixedcmtrecs++;
         } elsif ($_ =~ /^\,\,\,\,\,\,\,\,\,"/) {
            $v4 = 1;  # version 4 extended comment line
            if ($v4_convert) {
               # convert TLIB 4.12 journal file to 5.xx format
               $_ = ',,,' . $_;  # was 10 fields, now 13 fields
               $numV4converted++;
            }
         }
      } else {

         # Split into fields.  This isn't quite perfect, because some older versions
         # of TLIB might have left quote marks in the comment written as the last
         # field.
         @fields = &split_csv($_);
         $numfields = $#fields + 1;

         if (($numfields < 12) && (10 != $numfields)) {  # should be 10 fields (for TLIB 4.12) or 13 fields (for TLIB 5.xx)
             if ($verbose) {
                printf "Warning: line %d not in recognized format (%d fields):\n%s\n", $numrecs, $numfields, $_;
             }
         } else {
             # Okay, there are 10 or 12 or 13 fields.
             #  (Note: an embedded quote+comma in the final (comment) field might make
             # it appear that there are 14.  Recent TLIBs won't do that, but some
             # older versions of TLIB could.)

             $v4 = ($numfields < 12);  # True for TLIB 4.12, false for TLIB 5.xx

             if ($v4) {
                # 10 fields (from old TLIB 4.12)

                if ($v4_convert) {
                   # convert TLIB 4.12 journal file to 5.xx format
                   ($date,$time,$cmd,$locking,$userID,$lib,$src,$lok,$vnum,$cmt) = @fields;
                   $key = $src;
                   $proj = '';
                   $otherproj = '';
                   # add the three missing fields, to change it from v4.12 format to v5.xx format:
                   splice(@fields,9,0,$key,$proj,$otherproj);
                   $fixed = 1;
                   $numV4converted++;
                }

             } else {
                # 13 fields (from TLIB 5.00 or later), or 12 from buggy TLIB 5.53

                if (12 == $numfields) {
                   splice(@fields,6,0,$fields[8]);  # src <- key
                   $numfixedrecs++;
                   $fixed = 1;
                }

                ## Not really necessary
                # ($date,$time,$cmd,$locking,$userID,$lib,$src,$lok,$vnum,$key,$proj,$otherproj,$cmt) = @fields;
                # if (('' eq $key) && ('' ne $src)) {
                #    $key = $src;
                #    $fields[9] = $key;
                #    $fixed = 1;
                # } elsif (('' eq $src) && ('' ne $key)) {
                #    $src = $key;
                #    $fields[6] = $src;
                #    $fixed = 1;
                # }

             }
         }
         if ($fixed) {
            $_ = '';
            for ($i=0; $i <= $#fields; $i++) {
               if ($fields[$i] =~ /\,/) {
                  # field contains a comma, so gotta quote it
                  $_ .= ('"' . $fields[$i] . '"');
               } else {
                  $_ .= $fields[$i];
               }
               if ($i < $#fields) {
                  $_ .= ',';
               }
            }#for
         }
      }
   }
   # else leave blank lines alone (TLIB didn't put 'em in there, though!)
   print OUTP "$_\n";
}#while

close INP;
close OUTP;

print "$numrecs lines read from $infile\n";
if (0 == ($numfixedrecs + $numfixedcmtrecs + $numV4converted)) {
   print "No fixes were needed";
   if ($changed_only) {
      unlink $outfile;
      print ", $outfile deleted";
   }
   print ".\n";
} else {
   if ($numV4converted) {
      print "$numV4converted TLIB 4.12 (10-field) records were converted to TLIB 5 format.\n";
   }
   if ($numfixedcmtrecs) {
      print "$numfixedcmtrecs comment records were fixed (v5.00 bug).\n";
   }
   if ($numfixedrecs) {
      print "$numfixedrecs records w/ missing field #7 were fixed (v5.53 bug).\n";
   }
}

exit 0;

__END__
